'use strict';
const path = require('path')
const fs = require('fs')
const util = require('util')

const asyncReaddir = util.promisify(fs.readdir)
const asyncStat = util.promisify(fs.stat)
const asyncLstat = util.promisify(fs.lstat)

async function readDirR(dir, options = {}) {
    options.onlyDir = options.onlyDir || false
    options.exts = options.exts || []

    options.exts = options.exts.map(item => item.toLowerCase())

    let fullResult = []
    const files = await asyncReaddir(dir, { withFileTypes: true })
    for (let file of files) {
        const filepath = path.join(dir, file.name)
        if (file.isDirectory()) {
            if (options.onlyDir) {
                fullResult.push(filepath)
            }
            const files2 = await readDirR(filepath, options)
            fullResult.push(...files2)
        }
        else if (options.onlyDir) {
        }
        else {
            if (options.exts.length && !options.exts.includes(path.extname(filepath).toLowerCase())) {
                continue
            }
            fullResult.push(filepath)
        }
    }
    return fullResult
}

async function readDirR2(dir, options = {}, actionFn) {
    options.onlyDir = options.onlyDir || false
    options.exts = options.exts || []
    options.exts = options.exts.map(item => item.toLowerCase())

    const fullResult = []
    const readDirCurrent = new Set()
    readDirCurrent.add(dir)
    while (true) {
        for (const dir of readDirCurrent) {
            readDirCurrent.delete(dir)
            const files = await asyncReaddir(dir, { withFileTypes: true })
            for (let file of files) {
                const filepath = path.join(dir, file.name)
                if (file.isDirectory()) {
                    readDirCurrent.add(filepath)
                    if (options.onlyDir) {
                        if (actionFn) {
                            await actionFn(filepath)
                        }
                        fullResult.push(filepath)
                    }
                }
                else if (!options.onlyDir) {
                    if (!options.exts.length) {
                        if (actionFn) {
                            await actionFn(filepath)
                        }
                        fullResult.push(filepath)
                    }
                    else if (options.exts.length && options.exts.includes(path.extname(file.name).toLowerCase())) {
                        if (actionFn) {
                            await actionFn(filepath)
                        }
                        fullResult.push(filepath)
                    }
                }
            }
        }

        if (!readDirCurrent.size) {
            break
        }
    }

    return fullResult
}

module.exports = readDirR2