'use strict';
const readline = require('readline')
const fs = require('fs')
const path = require('path')
const childProcess = require('child_process')
const util = require('util')

const vips = `./soft/libs/vips/linux/vips`
const dir = `./imgs/`

const asyncExec = util.promisify(childProcess.exec)
const asyncRename = util.promisify(fs.rename)

main()

async function main() {
    const files = recurseReadDir(dir, ['.jpg', '.jpeg', '.png',])

    console.time('work time')
    for (let i = 0; i < files.length; i++) {
        let filepath = files[i]

        console.log(`${i + 1} / ${files.length}`)
        console.log(`convert to grayscale: ${filepath}`)
        console.log('\n')

        // let toRGB = `${convert} "${filepath}" -colorspace RGB -depth 24 PNG24:"${filepath}_1.png"`
        // await asyncExec(toRGB)

        // let toGray = `${convert} "${filepath}" -colorspace gray "${filepath}"`
        // await asyncExec(toGray)

        let tempName = filepath + '____53wertwregtsdfg34gdfzzpng.png'
        let toGray = `"${vips}" colourspace "${filepath}" "${tempName}" b-w`
        await asyncExec(toGray)
        await asyncRename(tempName, filepath)
    }
    console.timeEnd('work time')

    await askQuestion(`All ready`)
}

function askQuestion(query) {
    const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
    })

    return new Promise(resolve => rl.question(query, ans => {
        rl.close()
        resolve(ans)
    }))
}

function recurseReadDir(dir, types) {
    const fileList = []
    const files = fs.readdirSync(dir)
    for (let file of files) {
        const fullPath = path.join(dir, file)
        if (fs.statSync(fullPath).isDirectory()) {
            const innerFiles = recurseReadDir(fullPath, types)
            fileList.push(...innerFiles)
        }
        else if (fs.statSync(fullPath).isFile()) {
            if (~types.indexOf(path.extname(fullPath).toLowerCase())) {
                fileList.push(fullPath)
            }
        }
    }

    return fileList
}